/*
 *file name:sccb.cpp
 *description:sccbӿͨѶ.
 */
#include <windows.h>
#include <nkintr.h>
#include <ceddk.h>

#include "sccb.h"

#define uint32 unsigned int
#define uint16 unsigned short
#define uint8  unsigned char

#define CAM_WB_AUTO             0
#define	CAM_WB_CLOUD			1
#define	CAM_WB_DAYLIGHT			2
#define	CAM_WB_INCANDESCENCE	3
#define	CAM_WB_FLUORESCENT 	    4
#define	CAM_WB_TUNGSTEN			5
#define	CAM_WB_MANUAL           6
/* CAMERA PREVIEW FRAME RATE DEFINITION */
	#define CAM_PREVIEW_15FPS
//	#define CAM_PREVIEW_22FPS
//	#define CAM_PREVIEW_30FPS

	#define SYSTEM_CLK                           (48*1000*1000)
	/* PIXEL CLOCK USED BY BANDING FILTER CACULATION*/
#if defined(CAM_PREVIEW_15FPS)
	#define PIXEL_CLK							         (SYSTEM_CLK/8)		// 52/8 MHz
#elif defined(CAM_PREVIEW_22FPS)
    #define PIXEL_CLK							         (SYSTEM_CLK/6)		// 52/6 MHz
#elif defined(CAM_PREVIEW_30FPS)
    #define PIXEL_CLK 						      	     (SYSTEM_CLK/4)		// 52/4 MHz
#endif

#define BASEGAIN	0x40
/* SENSOR EXPOSURE LINE LIMITATION */
#define VGA_EXPOSURE_LIMITATION					510 //509//508
/* SENSOR VGA SIZE */
#define IMAGE_SENSOR_VGA_WIDTH                  (640)
#define IMAGE_SENSOR_VGA_HEIGHT                 (480)
/* SENSOR PIXEL/LINE NUMBERS IN ONE PERIOD */
#define VGA_PERIOD_PIXEL_NUMS                       784
#define VGA_PERIOD_LINE_NUMS                        510

uint16 dummy_lines=0;
uint16 sensor_frame_rate;
uint16 extra_exposure_lines=0;
int sensor_global_gain=BASEGAIN;
/*
 * ʼλ.
 */
void SCCB_START(void)
{
	SCCB_SET_SCL_OUT();
	SCCB_SET_SDA_OUT();
    SCCB_SCL_HIGH();
    SCCB_SDA_HIGH();
    
    SCCB_Delay();
	SCCB_SDA_LOW();
	SCCB_Delay();
	SCCB_SCL_LOW();
	SCCB_Delay();
}

/*
 * ֹͣλ.
 */
void SCCB_STOP(void)
{
    SCCB_SET_SCL_OUT();
	SCCB_SET_SDA_OUT();
    SCCB_Delay();
    SCCB_Delay();
    SCCB_SCL_LOW();
	SCCB_SDA_LOW();
	SCCB_Delay();
	SCCB_SCL_HIGH();
	SCCB_Delay();
	SCCB_SDA_HIGH();
	SCCB_Delay();	
    SCCB_Delay();
    SCCB_Delay();
    SCCB_Delay();
    SCCB_Delay();    
}

#define SCCB_9BIT_TIMEOUT (400000000/1000000)

/*
 * ȴ9λЧ,߳ʱ˳
 */
UCHAR SCCB_WAIT_READCOMPLETE(void)
{
    volatile DWORD timeout = SCCB_9BIT_TIMEOUT;
    
    while(1){
        if (0 == SCCB_SDA_READ())
            break; 
        if (!timeout--)
            break;
    }
    
    if (!timeout)
        return 1;
    else 
        return 0;
}

void NACK_BIT(void)
{
    SCCB_Delay();
    SCCB_SET_SDA_OUT();
    SCCB_Delay();
    SCCB_SDA_HIGH();
    SCCB_Delay();
    SCCB_SCL_HIGH();
    SCCB_Delay();
    SCCB_SCL_LOW();
    SCCB_Delay();        
}


/*************************************************************************
* FUNCTION
*	sccb_send_byte
*
* DESCRIPTION
*	This function send one byte to through software SCCB
*
* PARAMETERS
*	send_byte : the data that writes out through software sccb
*
* RETURNS
*	dwErr=0 success.
*   dwErr=1 timeout.
*
* GLOBALS AFFECTED
*
*************************************************************************/
DWORD sccb_send_byte(UCHAR send_byte)
{
	volatile signed char j;
    DWORD dwErr = 0;
    SCCB_SET_SDA_OUT();
	SCCB_Delay();
		
	for (j=7;j>=0;j--)	{
		if (send_byte & (1<<j))	{
			SCCB_SDA_HIGH();
		} else {
			SCCB_SDA_LOW();
		}
        
		SCCB_Delay();
		SCCB_SCL_HIGH();
		SCCB_Delay();
		SCCB_SCL_LOW();
        SCCB_Delay();
	}
    
	/* don't care bit, 9th bit */
	SCCB_SDA_LOW();
	SCCB_Delay();
    SCCB_SET_SDA_IN();
	SCCB_SCL_HIGH();

	SCCB_Delay();
    //dwErr = SCCB_WAIT_READCOMPLETE();
    SCCB_Delay();
	SCCB_SCL_LOW();
    SCCB_Delay();
    return dwErr;
}

/*************************************************************************
* FUNCTION
*	sccb_get_byte
*
* DESCRIPTION
*	This function read one byte through software SCCB interface
*
* PARAMETERS
*	None
*
* RETURNS
*	the data that read from SCCB interface
*
* GLOBALS AFFECTED
*
*************************************************************************/
UCHAR sccb_get_byte(void)
{
    UCHAR get_byte=0x00;
    volatile signed char i;
    //SCCB_SDA_LOW();
    SCCB_SET_SDA_IN();
    SCCB_Delay();
	for (i=7;i>=0;i--)
	{	// data bit 7~0 
		
        SCCB_SCL_HIGH();
        SCCB_Delay();
        if (SCCB_SDA_READ())
			get_byte |= (1<<i);
        SCCB_SCL_LOW();
        SCCB_Delay();
	}
	// don't care bit, 9th bit 
	//NACK_BIT();		
    SCCB_SDA_HIGH();
	SCCB_SET_SDA_OUT();
    SCCB_Delay();
    SCCB_SCL_HIGH();
    SCCB_Delay();
    SCCB_SCL_LOW();
    SCCB_Delay(); 
    return get_byte;
}

/*************************************************************************
* FUNCTION
*	sccb_write
*
* DESCRIPTION
*	This function 3 phase write to specified register through SCCB interface
*
* PARAMETERS
*	SubAddress : the register index of device
*   Value : setting value of the specified register of device
*
* RETURNS
*	None
*************************************************************************/
DWORD sccb_write(
    UCHAR SCCB_ID,      // slave  ID.
    UCHAR SubAddress,   // Register address.
    UCHAR Value)
{
    /* 3-phase write */
	SCCB_START();
	SCCB_Delay();
	sccb_send_byte(SCCB_ID);
	SCCB_Delay();
	sccb_send_byte(SubAddress);
	SCCB_Delay();
	sccb_send_byte(Value);
	SCCB_Delay();
	SCCB_STOP();
    return 0;
}

/*************************************************************************
* FUNCTION
*	sccb_read
*
* DESCRIPTION
*	This function read one byte through software SCCB
*
* PARAMETERS
*	SCCB_ID: slave ID.
*   subAddress:register address.
*
* RETURNS
*	byte from slave.
*
* GLOBALS AFFECTED
*
*************************************************************************/

UCHAR sccb_read(
    UCHAR SCCB_ID,      // slave  ID.
    UCHAR SubAddress    // Register address.
    )
{
    UCHAR get_byte;
    /* 2-phase write */
	SCCB_START();
	SCCB_Delay();
	//sccb_send_byte(SCCB_ID & ~0x01);
	sccb_send_byte(0x66);
	SCCB_Delay();
	sccb_send_byte(SubAddress);
	SCCB_Delay();
	SCCB_STOP();
    SCCB_Delay();
	SCCB_Delay();
    //SCCB_Delay();
    /* 2-phase read */
	    
	SCCB_START();
	SCCB_Delay();
	sccb_send_byte(0x67);
	SCCB_Delay();
	SCCB_Delay();
	SCCB_Delay();
	//sccb_send_byte(SCCB_ID | 0x01);
    //SCCB_Delay();
    SCCB_Delay();
	get_byte=sccb_get_byte();
	SCCB_Delay();
	SCCB_STOP();

    RETAILMSG(0,(TEXT("addr:%.2X  get:%.2X\r\n"), SubAddress,get_byte));
    return get_byte;
}

DWORD SCCB_Read(UCHAR SlaveAdd, UCHAR StartReg, PUCHAR pBuff, DWORD nRegs)
{

    for (;nRegs;nRegs--,pBuff++,StartReg++) {
        *pBuff = sccb_read(SlaveAdd, StartReg);
        RETAILMSG(0,(TEXT("SCCB_Read:slaveID:%.2X,value:%.2X,reg:%.2X\r\n"),\
            SlaveAdd, *pBuff, StartReg));
    }    

    return 0;
}

DWORD SCCB_Write(UCHAR SlaveAdd, UCHAR StartReg, PUCHAR pBuff, DWORD nRegs)
{

    for (;nRegs;nRegs--,pBuff++,StartReg++) {

        sccb_write(SlaveAdd, StartReg, *pBuff);

    }  

    return 0;
}





